(***********************************************************************)
(*                                                                     *)
(*                              OCamlGen                               *)
(*                                                                     *)
(*                       OCaml module Generator                        *)
(*                                                                     *)
(*               EPI Pomdapi, INRIA Paris - Rocquencourt               *)
(*                                                                     *)
(*  Copyright 1997-2014 INRIA.                                         *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*                                                                     *)
(***********************************************************************)

(* $Id: walk_spec.mli,v 1.11 2014/01/31 09:51:52 fclement Exp $ *)

(* Walk a type definition to generate matching functions for this type. *)

module type Out =
sig

  val buffer : Format.formatter;;

end
;;

module type Walk =

sig

  val begin_file : unit -> unit
  val end_file : unit -> unit

  val comment : string -> unit
  val directive : string -> unit

  val variable : Walker_ast.name -> unit

  val begin_paren : unit -> unit
  val end_paren : unit -> unit

  val begin_application : unit -> unit
    val separator_argument : unit -> unit
    val begin_argument : unit -> unit
    val end_argument : unit -> unit
  val end_application : unit -> unit

  val begin_tuple : unit -> unit
    val separator_tuple : unit -> unit
  val end_tuple : unit -> unit

  val begin_record_pattern : unit -> unit
    val begin_record_field : Walker_ast.record_label -> unit
    val end_record_field : unit -> unit
    val separator_record_field : unit -> unit
  val end_record_pattern : unit -> unit

  val begin_def : Walker_ast.rec_flag -> unit
    val begin_def_elem : (Og_types.type_name * Walker_ast.walker_name) * Walker_ast.poly_arguments -> unit
    val end_def_elem : unit -> unit
    val separator_def_elem : unit -> unit
  val end_def : unit -> unit

  val begin_function : unit -> unit
    val begin_clause_pattern : unit -> unit
    val end_clause_pattern : unit -> unit
    val begin_clause_expr : unit -> unit
    val end_clause_expr : unit -> unit
  val end_function : unit -> unit

  val sum_tag : Walker_ast.sum_label -> unit

  val begin_sum_inj : Walker_ast.sum_label -> unit
  val end_sum_inj : Walker_ast.sum_label -> unit

  val begin_record_clause : unit -> unit
  val end_record_clause : unit -> unit

  val begin_sum_clause : unit -> unit
    val begin_sum_field : unit -> unit
    val end_sum_field : unit -> unit
  val end_sum_clause : unit -> unit

  val begin_alias_clause : unit -> unit
    val begin_aliased_clause : unit -> unit
    val end_aliased_clause : unit -> unit
  val end_alias_clause : unit -> unit

end
;;

module type T = Walk
;;

module type Make =
  functor (Io : Out) -> Walk
;;
